#!/usr/bin/env python3

from math import cos, sin

from .miscs import deg2rad, rad2deg
from .space import G0, LANDER_WET_MASS, LANDER_DRY_MASS, MOON_RADIUS, MOON_ORBITAL_15000, moon_centrifugal, moon_gravity


class SpacecraftSim:
    """
    Simulate the spacecraft with SpacecraftSim.tick()
    """
    def __init__(self, time_forward=True, plotter=None):
        self.t = 0

        self.thrust = 45*1000                       # N = kg*m/s**2
        self.isp = 311                              # s
        self.mass_flow = self.thrust/(G0*self.isp)  # kg/s

        self.acc_x = 0                              # m/s**2
        self.acc_y = 0                              # m/s**2
        self.g = 0

        if time_forward:
            self.pos_x = 0                          # m
            self.pos_y = 15_000                     # m
            self.vel_x = MOON_ORBITAL_15000         # m/s
            self.vel_y = 0                          # m/s

            self.mass = LANDER_WET_MASS - 1000      # kg - Adjusted from Apollo
            self.mass_flow_direction = +1
        else:
            self.pos_x = 0                          # m
            self.pos_y = 0                          # m
            self.vel_x = 0                          # m/s
            self.vel_y = 0                          # m/s

            # Apollo: 250 kg ~= 60 sec ~= 5% (but Apollo was running at 60% thrust for a long time)
            self.mass = LANDER_DRY_MASS + 1000      # kg - Adjusted from Apollo
            self.mass_flow_direction = -1

        self.dv_flight = 0

        self.p = plotter

    def tick(self, thrust_ratio=1, angle=deg2rad(90), dt=1):
        """
        angle: spacecraft pointing. Engine will be opposite.
        """
        acc = self.thrust*thrust_ratio/self.mass
        self.acc_x = acc*cos(angle)
        self.acc_y = acc*sin(angle) - moon_gravity(self.pos_y) + moon_centrifugal(self.vel_x, self.pos_y)
        self.g = self.acc_y/G0

        self.vel_x += self.acc_x*dt
        self.vel_y += self.acc_y*dt

        self.pos_x += self.vel_x*dt
        self.pos_y += self.vel_y*dt

        self.mass -= self.mass_flow_direction*self.mass_flow*thrust_ratio*dt
        self.dv_flight += acc*dt
        self.t += dt

        if self.p:
            self.p.tick(self.t, self.pos_x, self.pos_y, self.vel_x, self.vel_y, self.acc_x, self.acc_y, angle, thrust_ratio)

    def print_header(self):
        print('         T |             Pos                   Vel                   Acc        | Angle |    G       Mass')

    def print_values(self):
        if (len(self.p.angle) % 50) == 0:
            self.print_header()

        print('%10.3f | %11.3f,%10.3f %10.3f,%10.3f %10.3f,%10.3f | %5.1f | %5.3f %10.3f' % (
            self.t, self.pos_x, self.pos_y, self.vel_x, self.vel_y, self.acc_x, self.acc_y,
            rad2deg(self.p.angle[-1]),
            self.g, self.mass,
        ))
